@@ -1,5 +1,5 @@
 // Signal to dynamically load the library
-#define ICSNEOC_DYNAMICLOAD
+//#define ICSNEOC_DYNAMICLOAD
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -30,13 +30,13 @@ void printAllDevices() {
 	if(numDevices == 0) {
 		printf("No devices found! Please scan for new devices.\n");
 	}
-	for(int i = 0; i < numDevices; i++) {
-		char productDescription[ICSNEO_DEVICETYPE_LONGEST_DESCRIPTION];
+	for(size_t i = 0; i < numDevices; i++) {
+		char productDescription[ICSNEO_DEVICETYPE_LONGEST_DESCRIPTION] = { 0 };
 		size_t descriptionLength = ICSNEO_DEVICETYPE_LONGEST_DESCRIPTION;
 
 		// Updates productDescription and descriptionLength for each device
 		if(icsneo_describeDevice(devices + i, productDescription, &descriptionLength)) {
-			printf("[%d] %s\tConnected: ", i + 1, productDescription);
+			printf("[%zd] %s\tConnected: ", i + 1, productDescription);
 			if(icsneo_isOpen(devices + i)) {
 				printf("Yes\t");
 			} else printf("No\t");
@@ -52,7 +52,7 @@ void printAllDevices() {
 			} else printf("Off\n");
 
 		} else {
-			printf("Description for device %d not available!\n", i + 1);
+			printf("Description for device %zd not available!\n", i + 1);
 		}
 	}
 }
@@ -66,7 +66,7 @@ size_t scanNewDevices() {
 	size_t numNewDevices = 99;
 	icsneo_findAllDevices(newDevices, &numNewDevices);
 
-	for(int i = 0; i < numNewDevices; ++i) {
+	for(size_t i = 0; i < numNewDevices; ++i) {
 		devices[numDevices + i] = newDevices[i];
 	}
 	numDevices += numNewDevices;
@@ -86,6 +86,8 @@ void printMainMenu() {
 	printf("H - Get events\n");
 	printf("I - Set HS CAN to 250K\n");
 	printf("J - Set HS CAN to 500K\n");
+	printf("L - Set Digital IO\n");
+	printf("M - Set HS CAN termination\n");
 	printf("X - Exit\n");
 }
 
@@ -110,7 +112,7 @@ void printAPIEvents() {
 			printf("Event 0x%u: %s\n", events[0].eventNumber, events[0].description);
 		} else {
 			printf("%d API events found!\n", (int) eventCount);
-			for(int i = 0; i < eventCount; ++i) {
+			for(size_t i = 0; i < eventCount; ++i) {
 				printf("Event 0x%u: %s\n", events[i].eventNumber, events[i].description);
 			}
 		}
@@ -126,13 +128,13 @@ void printAPIEvents() {
 void printDeviceEvents(neodevice_t* device) {
 	neoevent_t events[99];
 	size_t eventCount = 99;
-	if(icsneo_getDeviceEvents(selectedDevice, events, &eventCount)) {
+	if(icsneo_getDeviceEvents(device, events, &eventCount)) {
 		if(eventCount == 1) {
 			printf("1 device event found!\n");
 			printf("Event 0x%x: %s\n", events[0].eventNumber, events[0].description);
 		} else {
 			printf("%d device events found!\n", (int) eventCount);
-			for(int i = 0; i < eventCount; ++i) {
+			for(size_t i = 0; i < eventCount; ++i) {
 				printf("Event 0x%x: %s\n", events[i].eventNumber, events[i].description);
 			}
 		}
@@ -158,13 +160,15 @@ char getCharInput(int numArgs, ...) {
 
 	char* list = (char*) calloc(numArgs, sizeof(char));
 	for(int i = 0; i < numArgs; ++i) {
-		*(list + i) = va_arg(vaList, int);
+		*(list + i) = (char)va_arg(vaList, int);
 	}
 
 	va_end(vaList);
 
 	while(!found) {
-		fgets(input, 99, stdin);
+		if(fgets(input, 99, stdin) == NULL) {
+			break;
+		}
 		if(strlen(input) == 2) {
 			for(int i = 0; i < numArgs; ++i) {
 				if(input[0] == *(list + i)) {
@@ -194,10 +198,14 @@ const neodevice_t* selectDevice() {
 	printAllDevices();
 	printf("\n");
 
-	int selectedDeviceNum = 10;
+	size_t selectedDeviceNum = 10;
 
 	while(selectedDeviceNum > numDevices) {
 		char deviceSelection = getCharInput(9, '1', '2', '3', '4', '5', '6', '7', '8', '9');
+		if(deviceSelection < '0') {
+			printf("Selected device out of range!\n");
+			continue;
+		}
 		selectedDeviceNum = deviceSelection - '0';
 		if(selectedDeviceNum > numDevices) {
 			printf("Selected device out of range!\n");
@@ -213,21 +221,21 @@ int main() {
 
 	// Attempt to initialize the library and access its functions
 	// This call searches for icsneoc.dll according to the standard dynamic-link library search order
-	int ret = icsneo_init();
-	if(ret == 1) {
-		printf("The library was already initialized!\n");
-		return ret;
-	}
-
-	if(ret == 2) {
-		printf("The library could not be found!\n");
-		return ret;
-	}
-
-	if(ret == 3) {
-		printf("The library is missing functions!\n");
-		return ret;
-	}
+	//int ret = icsneo_init();
+	//if(ret == 1) {
+	//	printf("The library was already initialized!\n");
+	//	return ret;
+	//}
+//
+	//if(ret == 2) {
+	//	printf("The library could not be found!\n");
+	//	return ret;
+	//}
+//
+	//if(ret == 3) {
+	//	printf("The library is missing functions!\n");
+	//	return ret;
+	//}
 
 	neoversion_t ver = icsneo_getVersion();
 	printf("ICS icsneoc.dll version %u.%u.%u\n\n", ver.major, ver.minor, ver.patch);
@@ -235,7 +243,7 @@ int main() {
 	while(true) {
 		printMainMenu();
 		printf("\n");
-		char input = getCharInput(22, 'A', 'a', 'B', 'b', 'C', 'c', 'D', 'd', 'E', 'e', 'F', 'f', 'G', 'g', 'H', 'h', 'I', 'i', 'J', 'j', 'X', 'x');
+		char input = getCharInput(26, 'A', 'a', 'B', 'b', 'C', 'c', 'D', 'd', 'E', 'e', 'F', 'f', 'G', 'g', 'H', 'h', 'I', 'i', 'J', 'j', 'L', 'l', 'M', 'm', 'X', 'x');
 		printf("\n");
 		switch(input) {
 		// List current devices
@@ -270,17 +278,17 @@ int main() {
 			selectedDevice = selectDevice();
 
 			// Get the product description for the device
-			char productDescription[ICSNEO_DEVICETYPE_LONGEST_DESCRIPTION];
+			char productDescription[ICSNEO_DEVICETYPE_LONGEST_DESCRIPTION] = { 0 };
 			size_t descriptionLength = ICSNEO_DEVICETYPE_LONGEST_DESCRIPTION;
 			icsneo_describeDevice(selectedDevice, productDescription, &descriptionLength);
 
 			printf("Would you like to open or close %s?\n", productDescription);
 			printf("[1] Open\n[2] Close\n[3] Cancel\n\n");
 
-			char input = getCharInput(3, '1', '2', '3');
+			char userSelection = getCharInput(3, '1', '2', '3');
 			printf("\n");
 
-			switch(input) {
+			switch(userSelection) {
 			case '1':
 				// Attempt to open the selected device
 				if(icsneo_openDevice(selectedDevice)) {
@@ -299,7 +307,7 @@ int main() {
 
 					// Shifts everything after the removed device 1 index to the left
 					bool startResizing = false;
-					for(int i = 0; i < numDevices; ++i) {
+					for(size_t i = 0; i < numDevices; ++i) {
 						if(selectedDevice == devices + i)
 							startResizing = true;
 						if(startResizing)
@@ -312,7 +320,7 @@ int main() {
 					printLastError();
 					printf("\n");
 				}
-				
+
 				break;
 			default:
 				printf("Canceling!\n\n");
@@ -331,17 +339,17 @@ int main() {
 			selectedDevice = selectDevice();
 
 			// Get the product description for the device
-			char productDescription[ICSNEO_DEVICETYPE_LONGEST_DESCRIPTION];
+			char productDescription[ICSNEO_DEVICETYPE_LONGEST_DESCRIPTION] = { 0 };
 			size_t descriptionLength = ICSNEO_DEVICETYPE_LONGEST_DESCRIPTION;
 			icsneo_describeDevice(selectedDevice, productDescription, &descriptionLength);
 
 			printf("Would you like to have %s go online or offline?\n", productDescription);
 			printf("[1] Online\n[2] Offline\n[3] Cancel\n\n");
 
-			char input = getCharInput(3, '1', '2', '3');
+			char userSelection = getCharInput(3, '1', '2', '3');
 			printf("\n");
 
-			switch(input) {
+			switch(userSelection) {
 			case '1':
 				// Attempt to go online
 				if(icsneo_goOnline(selectedDevice)) {
@@ -379,17 +387,17 @@ int main() {
 			selectedDevice = selectDevice();
 
 			// Get the product description for the device
-			char productDescription[ICSNEO_DEVICETYPE_LONGEST_DESCRIPTION];
+			char productDescription[ICSNEO_DEVICETYPE_LONGEST_DESCRIPTION] = { 0 };
 			size_t descriptionLength = ICSNEO_DEVICETYPE_LONGEST_DESCRIPTION;
 			icsneo_describeDevice(selectedDevice, productDescription, &descriptionLength);
 
 			printf("Would you like to enable or disable message polling for %s?\n", productDescription);
 			printf("[1] Enable\n[2] Disable\n[3] Cancel\n\n");
 
-			char input = getCharInput(3, '1', '2', '3');
+			char userSelection = getCharInput(3, '1', '2', '3');
 			printf("\n");
 
-			switch(input) {
+			switch(userSelection) {
 			case '1':
 				// Attempt to enable message polling
 				if(icsneo_enableMessagePolling(selectedDevice)) {
@@ -423,7 +431,7 @@ int main() {
 				break;
 			default:
 				printf("Canceling!\n\n");
-			}	
+			}
 		}
 		break;
 		// Get messages
@@ -438,12 +446,12 @@ int main() {
 			selectedDevice = selectDevice();
 
 			// Get the product description for the device
-			char productDescription[ICSNEO_DEVICETYPE_LONGEST_DESCRIPTION];
+			char productDescription[ICSNEO_DEVICETYPE_LONGEST_DESCRIPTION] = { 0 };
 			size_t descriptionLength = ICSNEO_DEVICETYPE_LONGEST_DESCRIPTION;
 			icsneo_describeDevice(selectedDevice, productDescription, &descriptionLength);
 
 			// Prepare the array of neomessage_t ptrs for reading in the messages
-			neomessage_t* msgs = (neomessage_t*) malloc(msgLimit * sizeof(neomessage_t*));
+			neomessage_t* msgs = (neomessage_t*) malloc(msgLimit * sizeof(neomessage_t));
 
 			// Get messages
 			size_t msgCount = msgLimit;
@@ -465,22 +473,59 @@ int main() {
 
 			// Print out the received messages
 			for(size_t i = 0; i < msgCount; i++) {
-				neomessage_t* msg = &msgs[i];
-				switch(msg->type) {
-				case ICSNEO_NETWORK_TYPE_CAN: // CAN 
-				{
-					neomessage_can_t* canMsg = (neomessage_can_t*) msg;
-					printf("\t0x%03x [%zu] ", canMsg->arbid, canMsg->length);
-					for(size_t i = 0; i < canMsg->length; i++) {
-						printf("%02x ", canMsg->data[i]);
+				const neomessage_t* msg = &msgs[i];
+				switch(msg->messageType) {
+					case ICSNEO_MESSAGE_TYPE_FRAME: {
+						const neomessage_frame_t* frame = (neomessage_frame_t*)msg;
+						switch(frame->type) {
+							case ICSNEO_NETWORK_TYPE_CAN: {
+								neomessage_can_t* canMsg = (neomessage_can_t*)frame;
+								printf("\t0x%03x [%zu] ", canMsg->arbid, canMsg->length);
+								for(size_t j = 0; j < canMsg->length; j++) {
+									printf("%02x ", canMsg->data[j]);
+								}
+								if(canMsg->status.transmitMessage)
+									printf("TX%s %04x ", canMsg->status.globalError ? " ERR" : "", canMsg->description);
+								printf("(%"PRIu64")\n", canMsg->timestamp);
+								break;
+							}
+							case ICSNEO_NETWORK_TYPE_LIN: {
+								neomessage_lin_t* linMsg = (neomessage_lin_t*)frame;
+								size_t frameLen = linMsg->length;
+								size_t dataLen = (frameLen > 2) ? (frameLen - 2) : 0;
+								size_t numberBytesHeader = (dataLen > 1) ? 3 : 1;
+								size_t numberBytesData = frameLen - numberBytesHeader;
+								if(linMsg->netid == ICSNEO_NETID_LIN_01) {
+									printf("LIN 1 | ID: 0x%02x [%zu] ", linMsg->header[0], dataLen);
+								}
+								else if (linMsg->netid == ICSNEO_NETID_LIN_02) {
+									printf("LIN 2 | ID: 0x%02x [%zu] ", linMsg->header[0], dataLen);
+								}
+
+								for(size_t j = 0; j < dataLen; ++j) {
+									if (j < 2) {
+										printf("%02x ", linMsg->header[j+1]);
+									} else {
+										printf("%02x ", linMsg->data[j-2]);
+									}
+								}
+								if(numberBytesData > 0)
+									printf("| Checksum: 0x%02x\n", linMsg->data[numberBytesData-1]);
+								else
+									printf("| Checksum: 0x%02x\n", linMsg->header[numberBytesHeader-1]);
+							}
+							default:
+								printf("\tMessage on netid %d with length %zu\n", frame->netid, frame->length);
+								break;
+						}
+						break;
+					}
+					case ICSNEO_MESSAGE_TYPE_CAN_ERROR_COUNT: {
+						const neomessage_can_error_t* cec = (neomessage_can_error_t*)msg;
+						printf("\tCAN error counts changed, TEC=%d, REC=%d%s", cec->transmitErrorCount, cec->receiveErrorCount,
+							cec->status.canBusOff ? " (Bus Off)" : "");
+						break;
 					}
-					printf("(%"PRIu64")\n", canMsg->timestamp);
-					break;
-				}
-				default:
-					if(msg->netid != 0)
-						printf("\tMessage on netid %d with length %zu\n", msg->netid, msg->length);
-					break;
 				}
 			}
 			printf("\n");
@@ -501,7 +546,7 @@ int main() {
 			selectedDevice = selectDevice();
 
 			// Get the product description for the device
-			char productDescription[ICSNEO_DEVICETYPE_LONGEST_DESCRIPTION];
+			char productDescription[ICSNEO_DEVICETYPE_LONGEST_DESCRIPTION] = { 0 };
 			size_t descriptionLength = ICSNEO_DEVICETYPE_LONGEST_DESCRIPTION;
 			icsneo_describeDevice(selectedDevice, productDescription, &descriptionLength);
 
@@ -517,8 +562,9 @@ int main() {
 			neomessage_can_t msg = {0};
 			msg.arbid = 0x120;
 			msg.length = 6;
-			msg.netid = ICSNEO_NETID_HSCAN;
+			msg.netid = ICSNEO_NETID_DWCAN_01;
 			msg.data = sendMessageData;
+			msg.description = 0x1c5; // Random description to match the receipt
 			msg.status.canfdFDF = false;
 			msg.status.extendedFrame = false;
 			msg.status.canfdBRS = false;
@@ -555,12 +601,12 @@ int main() {
 			selectedDevice = selectDevice();
 
 			// Get the product description for the device
-			char productDescription[ICSNEO_DEVICETYPE_LONGEST_DESCRIPTION];
+			char productDescription[ICSNEO_DEVICETYPE_LONGEST_DESCRIPTION] = { 0 };
 			size_t descriptionLength = ICSNEO_DEVICETYPE_LONGEST_DESCRIPTION;
 			icsneo_describeDevice(selectedDevice, productDescription, &descriptionLength);
 
 			// Attempt to set baudrate and apply settings
-			if(icsneo_setBaudrate(selectedDevice, ICSNEO_NETID_HSCAN, 250000) && icsneo_settingsApply(selectedDevice)) {
+			if(icsneo_setBaudrate(selectedDevice, ICSNEO_NETID_DWCAN_01, 250000) && icsneo_settingsApply(selectedDevice)) {
 				printf("Successfully set HS CAN baudrate for %s to 250k!\n\n", productDescription);
 			} else {
 				printf("Failed to set HS CAN for %s to 250k!\n\n", productDescription);
@@ -581,12 +627,12 @@ int main() {
 			selectedDevice = selectDevice();
 
 			// Get the product description for the device
-			char productDescription[ICSNEO_DEVICETYPE_LONGEST_DESCRIPTION];
+			char productDescription[ICSNEO_DEVICETYPE_LONGEST_DESCRIPTION] = { 0 };
 			size_t descriptionLength = ICSNEO_DEVICETYPE_LONGEST_DESCRIPTION;
 			icsneo_describeDevice(selectedDevice, productDescription, &descriptionLength);
 
 			// Attempt to set baudrate and apply settings
-			if(icsneo_setBaudrate(selectedDevice, ICSNEO_NETID_HSCAN, 500000) && icsneo_settingsApply(selectedDevice)) {
+			if(icsneo_setBaudrate(selectedDevice, ICSNEO_NETID_DWCAN_01, 500000) && icsneo_settingsApply(selectedDevice)) {
 				printf("Successfully set HS CAN baudrate for %s to 500k!\n\n", productDescription);
 			} else {
 				printf("Failed to set HS CAN for %s to 500k!\n\n", productDescription);
@@ -595,11 +641,155 @@ int main() {
 			}
 		}
 		break;
+		// Set Digital IO
+		case 'L':
+		case 'l':
+		{
+			// Select a device and get its description
+			if(numDevices == 0) {
+				printf("No devices found! Please scan for new devices.\n\n");
+				break;
+			}
+			selectedDevice = selectDevice();
+
+			printf("Select from the following:\n");
+
+			printf("[1] Ethernet (DoIP) Activation Line");
+			bool val;
+			bool haveVal = icsneo_getDigitalIO(selectedDevice, ICSNEO_IO_ETH_ACTIVATION, 1, &val);
+			if(!haveVal) {
+				neoevent_t event;
+				bool got = icsneo_getLastError(&event);
+				printf(": Unknown (%s)\n", got ? event.description : "No error");
+			} else {
+				if(val)
+					printf(": Enabled\n");
+				else
+					printf(": Disabled\n");
+			}
+
+			printf("[2] USB Host Power");
+			haveVal = icsneo_getDigitalIO(selectedDevice, ICSNEO_IO_USB_HOST_POWER, 1, &val);
+			if(!haveVal) {
+				neoevent_t event;
+				bool got = icsneo_getLastError(&event);
+				printf(": Unknown (%s)\n", got ? event.description : "No error");
+			} else {
+				if(val)
+					printf(": Enabled\n");
+				else
+					printf(": Disabled\n");
+			}
+
+			printf("[3] Backup Power");
+			haveVal = icsneo_getDigitalIO(selectedDevice, ICSNEO_IO_BACKUP_POWER_EN, 1, &val);
+			if(!haveVal) {
+				neoevent_t event;
+				bool got = icsneo_getLastError(&event);
+				printf(": Unknown (%s)\n", got ? event.description : "No error");
+			} else {
+				if(val)
+					printf(": Enabled ");
+				else
+					printf(": Disabled ");
+
+				haveVal = icsneo_getDigitalIO(selectedDevice, ICSNEO_IO_BACKUP_POWER_GOOD, 1, &val);
+				if(!haveVal) {
+					neoevent_t event;
+					bool got = icsneo_getLastError(&event);
+					printf("with unknown status (%s)\n", got ? event.description : "No error");
+				} else {
+					if(val)
+						printf("and Charged\n");
+					else
+						printf("and Not Charged\n");
+				}
+			}
+
+			printf("[4] Cancel\n\n");
+			char selection = getCharInput(4, '1', '2', '3', '4');
+			printf("\n");
+
+			if(selection == '4') {
+				printf("Canceling!\n\n");
+				break;
+			}
+
+			printf("[0] Disable\n[1] Enable\n[2] Cancel\n\n");
+			char selection2 = getCharInput(3, '0', '1', '2');
+			printf("\n");
+
+			if(selection2 == '2') {
+				printf("Canceling!\n\n");
+				break;
+			}
+
+			const bool set = selection2 == '1';
+			neoio_t type = ICSNEO_IO_ETH_ACTIVATION;
+			switch (selection)
+			{
+			case '1': type = ICSNEO_IO_ETH_ACTIVATION; break;
+			case '2': type = ICSNEO_IO_USB_HOST_POWER; break;
+			case '3': type = ICSNEO_IO_BACKUP_POWER_EN; break;
+			};
+			if(icsneo_setDigitalIO(selectedDevice, type, 1, set)) {
+				printf("OK!\n\n");
+			} else {
+				neoevent_t event;
+				bool got = icsneo_getLastError(&event);
+				printf("Failure! (%s)\n\n", got ? event.description : "No error");
+			}
+		}
+		break;
+		// Set HS CAN termination
+		case 'M':
+		case 'm':
+		{
+			// Select a device and get its description
+			if(numDevices == 0) {
+				printf("No devices found! Please scan for new devices.\n\n");
+				break;
+			}
+			selectedDevice = selectDevice();
+
+			// Get the product description for the device
+			char productDescription[ICSNEO_DEVICETYPE_LONGEST_DESCRIPTION] = { 0 };
+			size_t descriptionLength = ICSNEO_DEVICETYPE_LONGEST_DESCRIPTION;
+			icsneo_describeDevice(selectedDevice, productDescription, &descriptionLength);
+
+			printf("Termination is ");
+			const bool val = icsneo_isTerminationEnabledFor(selectedDevice, ICSNEO_NETID_DWCAN_01);
+			neoevent_t err = { 0 };
+			if(icsneo_getLastError(&err)) {
+				printf("not available at this time: %s\n\n", err.description);
+				break;
+			}
+			printf(val ? "currently enabled\n" : "currently disabled\n");
+
+			printf("[0] Disable\n[1] Enable\n[2] Cancel\n\n");
+			char selection2 = getCharInput(3, '0', '1', '2');
+			printf("\n");
+
+			if(selection2 == '2') {
+				printf("Canceling!\n\n");
+				break;
+			}
+
+			// Attempt to set baudrate and apply settings
+			if(icsneo_setTerminationFor(selectedDevice, ICSNEO_NETID_DWCAN_01, selection2 == '1') && icsneo_settingsApply(selectedDevice)) {
+				printf("Successfully set HS CAN termination for %s!\n\n", productDescription);
+			} else {
+				printf("Failed to set HS CAN termination for %s!\n\n", productDescription);
+				printLastError();
+				printf("\n");
+			}
+		}
+		break;
 		// Exit
 		case 'X':
 		case 'x':
 			printf("Exiting program\n");
-			return !icsneo_close();
+			return 0;
 		default:
 			printf("Unexpected input, exiting!\n");
 			return 1;
