@@ -1,32 +1,28 @@
-#ifndef __VALUECAN4_4_H_
-#define __VALUECAN4_4_H_
+#ifndef __VALUECAN4_2_H_
+#define __VALUECAN4_2_H_
 
 #ifdef __cplusplus
 
 #include "icsneo/device/tree/valuecan4/valuecan4.h"
-#include "icsneo/device/tree/valuecan4/settings/valuecan4-4settings.h"
+#include "icsneo/device/tree/valuecan4/settings/valuecan4-2settings.h"
 
 namespace icsneo {
 
-class ValueCAN4_4 : public ValueCAN4 {
+class ValueCAN4_2 : public ValueCAN4 {
 public:
-	// Serial numbers start with V4 for 4-4
+	// Serial numbers start with V2 for 4-2
 	// USB PID is 0x1101 (shared by all ValueCAN 4s), standard driver is CDCACM
-	ICSNEO_FINDABLE_DEVICE(ValueCAN4_4, DeviceType::VCAN4_4, "V4");
+	ICSNEO_FINDABLE_DEVICE(ValueCAN4_2, DeviceType::VCAN4_2, "V2");
 
 	enum class SKU {
 		Standard,
-		AP0400A_D26, // HDB26, USB A, and Keysight Branding
-		AP0400A_DB9, // 4xDB9, USB A, and Keysight Branding
-		AP0400A_OBD, // OBD, USB A, and Keysight Branding
+		AP0200A, // USB A and Keysight Branding
 	};
 
 	static const std::vector<Network>& GetSupportedNetworks() {
 		static std::vector<Network> supportedNetworks = {
 			Network::NetID::DWCAN_01,
-			Network::NetID::DWCAN_02,
-			Network::NetID::DWCAN_03,
-			Network::NetID::DWCAN_04
+			Network::NetID::DWCAN_02
 		};
 		return supportedNetworks;
 	}
@@ -34,11 +30,8 @@ public:
 	SKU getSKU() const {
 		switch(getSerial().back()) {
 			case 'A':
-				return SKU::AP0400A_D26;
 			case 'B':
-				return SKU::AP0400A_DB9;
-			case 'C':
-				return SKU::AP0400A_OBD;
+				return SKU::AP0200A;
 			default:
 				return SKU::Standard;
 		}
@@ -47,19 +40,15 @@ public:
 	std::string getProductName() const override {
 		switch(getSKU()) {
 			case SKU::Standard: break;
-			case SKU::AP0400A_D26:
-				return "Keysight AP0400A-D26";
-			case SKU::AP0400A_DB9:
-				return "Keysight AP0400A-DB9";
-			case SKU::AP0400A_OBD:
-				return "Keysight AP0400A-OBD";
+			case SKU::AP0200A:
+				return "Keysight AP0200A";
 		}
 		return Device::getProductName();
 	}
 
 protected:
-	ValueCAN4_4(neodevice_t neodevice, const driver_factory_t& makeDriver) : ValueCAN4(neodevice) {
-		initialize<ValueCAN4_4Settings>(makeDriver);
+	ValueCAN4_2(neodevice_t neodevice, const driver_factory_t& makeDriver) : ValueCAN4(neodevice) {
+		initialize<ValueCAN4_2Settings>(makeDriver);
 	}
 
 	void setupSupportedRXNetworks(std::vector<Network>& rxNetworks) override {
